package com.nisum.offertransactionservice.controller;

import com.github.tomakehurst.wiremock.core.WireMockConfiguration;
import com.github.tomakehurst.wiremock.junit.WireMockClassRule;
import com.netflix.loadbalancer.Server;
import com.netflix.loadbalancer.ServerList;
import com.nisum.offertransactionservice.OffertransactionserviceApplication;
import com.nisum.offertransactionservice.dto.*;
import com.nisum.offertransactionservice.model.OfferLookup;
import org.junit.ClassRule;
import org.junit.jupiter.api.*;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.TestConfiguration;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.cloud.contract.wiremock.AutoConfigureWireMock;
import org.springframework.cloud.netflix.ribbon.StaticServerList;
import org.springframework.context.annotation.Bean;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static com.github.tomakehurst.wiremock.client.WireMock.*;


@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment= SpringBootTest.WebEnvironment.DEFINED_PORT,classes = OffertransactionserviceApplication.class)
@AutoConfigureWireMock(port = 0)
@ActiveProfiles(value = "integration")
@ContextConfiguration(classes = {OfferTransactionControllerIntegrationTest.LocalRibbonClientConfiguration.class})
public class OfferTransactionControllerIntegrationTest{

    @ClassRule
    public static WireMockClassRule mockClassRule = new WireMockClassRule(WireMockConfiguration.options().dynamicPort());

    @Autowired
    private TestRestTemplate testRestTemplate;

    @BeforeAll
    public  static void beforeAll(){
        mockClassRule.start();
    }

    @AfterAll
    public  static void afterAll() {
        mockClassRule.stop();
    }


    @TestConfiguration
    public static class LocalRibbonClientConfiguration {

        @Bean
        public ServerList<Server> ribbonServerList() {
            return new StaticServerList<>(new Server("localhost", mockClassRule.port()));
        }
    }


    private void mockForOfferTransactionCall() {
        mockClassRule.stubFor(post(urlEqualTo("/promotionEngine/calculateDiscount"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withHeader("Content-Type", "application/json")
                        .withBodyFile("response.json")
                ));
    }

    @Test
    public void getOfferTransactionResponseTest() {
        mockForOfferTransactionCall();
        URI uri = URI.create("http://localhost:7072/offer-service/offerTransactionCall");
        RequestEntity requestEntity = new RequestEntity(getOfferTransactionRequest(),HttpMethod.POST,uri);

        ResponseEntity<OfferTransactionResponse> responseEntity = testRestTemplate.exchange("http://localhost:7072/offer-service/offerTransactionCall", HttpMethod.POST,requestEntity,
                new ParameterizedTypeReference<OfferTransactionResponse>(){}
        );
        OfferTransactionResponse offerTransactionResponse = responseEntity.getBody();

        Assertions.assertNotNull(offerTransactionResponse.getTransactionId());
        Assertions.assertEquals(2,offerTransactionResponse.getDiscountedItemList().size());
    }

    @Test
    public void endOfSaleTest() {
        URI uri = URI.create("http://localhost:7072/offer-service/endOfSale");
        RequestEntity requestEntity = new RequestEntity(getEndOfSaleReq(),HttpMethod.POST,uri);

        ResponseEntity<EndOfSaleResponse> responseEntity = testRestTemplate.exchange("http://localhost:7072/offer-service/endOfSale", HttpMethod.POST,requestEntity,
                new ParameterizedTypeReference<EndOfSaleResponse>(){}
        );
        EndOfSaleResponse endOfSaleResponse = responseEntity.getBody();
        Assertions.assertNotNull(endOfSaleResponse.getUuId());

    }

    private void mockForEndOfTransaction() {
        mockClassRule.stubFor(post(urlEqualTo("/store/producer"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withHeader("Content-Type", "application/json")
                        .withBody("Success from Producer")
                ));
    }
    @Test
    public void endOfTransactionTest() {
        mockForEndOfTransaction();
        URI uri = URI.create("http://localhost:7072/offer-service/endOfTransaction");
        String uuid = "86fd4146-0540-405b-b621-a95f4ccdfa0d";
        RequestEntity requestEntity = new RequestEntity(uuid,HttpMethod.POST,uri);

        ResponseEntity<String> responseEntity = testRestTemplate.exchange("http://localhost:7072/offer-service/endOfTransaction", HttpMethod.POST,requestEntity,
                new ParameterizedTypeReference<String>(){}
        );
        String endOfTransactionRes = responseEntity.getBody();
        Assertions.assertNotNull(endOfTransactionRes);
        Assertions.assertEquals("Success from Producer",endOfTransactionRes);

    }

    private EndOfSaleReq getEndOfSaleReq() {
        EndOfSaleReq endOfSaleReq = new EndOfSaleReq();
        endOfSaleReq.setOfferTransactionResponse(retOfferTransRes());
        endOfSaleReq.setOffers(returnOffers());
        return endOfSaleReq;
    }


    private OfferTransactionResponse retOfferTransRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        offerTransactionResponse.setTransactionId("123456788899999998");
        return offerTransactionResponse;
    }



    public List<OfferLookup> returnOffers() {
        OfferLookup offerLookup1 = new OfferLookup("1", "12", 23L, "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup("2", "13", 24L, "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups.stream().collect(Collectors.toList());
    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId("54321");
        offerTransactionRequest.setItemList(getItemList());
        return offerTransactionRequest;
    }

    private List<Item> getItemList() {
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05);
        Item item2 = new Item("Thumpsup", "11", 14.05);
        itemList.add(item1);
        itemList.add(item2);
        return itemList;
    }
}
